# Implementarea ferestrei glisante

Fereastra glisantă poate fi implementată sub forma unei liste simplu înlănțuite.
Această listă va conține inițial toate segmentele. Pentru a marca ultimul (cel mai recent adăugat) segment din fereastră vom folosi atributul `max_seq`, iar segmentele ieșite din fereastra glisantă vor fi șterse din listă. 

Astfel, în implementarea noastră de Go-Back-N ARQ:
- Primul element din fereastră va fi primul din listă;
- Ultimul element din fereastră, va fi ultimul element care are `seq <= max_seq`.

Pentru implementarea efectivă în C, în scheletul de cod aveți definite următoarele structuri/funcții:

```c
/* List entry */
struct cel {
  void *info;
  int info_len;
  int seq;
  char type;
  struct cel *next;
};

typedef struct cel list_entry;

/* Window as a list */
typedef struct {
  int size;
  int max_seq;
  list_entry *head;
} list;

/* Creates an empty list */
list *create_list();

/* Adds a segment to the window */
void add_list_elem(list *window, void *segment, int segment_size, int seq);
```

În lifecycle-ul protocolului, putem regăsi 3 tipuri de pachete:
- pachete ieșite din fereastra glisantă (pentru care am primit ACK);
- pachete trimise pentru care nu am primit încă ACK (in-flight);
- pachete care urmează a fi trimise (next-in-sequence), astfel încât `count(in-flight) + count(next-in-sequence)` să fie exact dimensiunea ferestrei noastre (măsurată în segmente).

Vizual, implementarea arată astfel:

![](images/window_representation.png)

> Slot-urile gri pot conține sau nu segmente.

În continuare, descriem câteva funcții utile în implementarea Go-Back-N ARQ. 

Vom presupune că am determinat `window_size` în funcție de BDP.
Atât la început, cât și la retransmisie, în implementarea de Go-Back-N vom
transmite toată fereastra.

```c
/* Functia trimite toate pachetele din fereastra */
void send_window(int sockfd, struct sockaddr *serveraddr, socklen_t serveraddr_len, list *window) {
    list_entry* e = window->head;
    while(e != NULL && e->seq <= window->max_seq) {
      sendto(sockfd, e->info, e->info_len, 0, serveraddr, serveraddr_len);
      e = e->next;
    }
}
```

Atunci când primim un ACK, vom marca toate mesajele cu `packet.seq <= secvența_mesajului_confirmat` ca fiind confirmate, prin ștergerea lor din listă. Aceste segmente nu ne mai folosesc, deci nu uităm să eliberăm memoria aferentă lor.

Întrucât ne este util în implementare, funcția va întoarce **câte elemente a eliminat din listă**.

```c
/* Scoate din lista segmentele cu segment.seq < seq. 
   Intoarce câte elemente a eliminat din lista.
*/
int update_left_window(list *window, int seq) {
    list_entry* e = window->head;
    int count = 0;

    while(e != NULL && e->seq <= seq) {
        list_entry *prev = e;
        e = e->next;
        // Eliberam memoria atat pentru informatia din segment, cat si pentru
        // nodul de lista
        free(prev->info);
        free(prev);
        count++;
    }

    window->head = e;
    return count;
}
```

Atunci când primim ACK pentru noi pachete, din fereastră se eliberează `count` poziții (câte întoarce funcția de mai sus) și trebuie să completăm fereastra cu noi segmente. Pentru asta vom folosi funcția de mai jos care adaugă în fereastră (și trimite) segmentele noi (`new-in-sequence`) până la `seq == new_max_seq` inclusiv.

```c
/* Adauga in fereastra (si trimite) segmentele noi până la seq == new_max_seq */
void send_new_segments(int sockfd, struct sockaddr* serveraddr, socklen_t serveraddr_len, list *window, int new_max_seq) {
    list_entry* e = window->head;
    /* Sarim peste segmentele deja trimise */
    while(e != NULL && e->seq <= window->max_seq) {
        e = e->next;
    }

    if (e != NULL) {
        /* Trimitem urmatoarele segmente pana la new_max_seq */
        while(e != NULL && e->seq <= new_max_seq) {
            sendto(sockfd, e->info, e->info_len, 0, serveraddr, serveraddr_len);
            e = e->next;
        }
    }

    window->max_seq = new_max_seq;
}
```
